import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';

export interface volumeInfo {
  title: string;
  authors: Array<string>;
  publisher: string;
  publishedDate: number;
  readingModes: {
    text: boolean;
    image: boolean;
  };
  pageCount: number;
  printType: string;
  categories: Array<string>;
  maturityRating: string;
  allowAnonLogging: boolean;
  contentVersion: string;
  panelizationSummary: {
    containsEpubBubbles: boolean;
    containsImageBubbles: boolean;
  };
  imageLinks: {
    smallThumbnail?: string;
    thumbnail?: string;
  };
  language: string;
  previewLink: string;
  infoLink: string;
  canonicalVolumeLink: string;
}

export interface saleInfo {
  country: string;
  saleability: string;
  isEbook: boolean;
  buyLink: string;
}

export interface accessInfo {
  country: string;
  viewability: string;
  embeddable: boolean;
  publicDomain: boolean;
  textToSpeechPermission: string;
  epub: {
    isAvailable: boolean;
    downloadLink?: string;
    acsTokenLink?: string;
  };
  pdf: {
    isAvailable: boolean;
    downloadLink?: string;
    acsTokenLink?: string;
  };
  webReaderLink: string;
  accessViewStatus: string;
  quoteSharingAllowed: boolean;
}

export interface Book {
  kind: string;
  id: string;
  etag: string;
  selfLink: string;
  volumeInfo: volumeInfo;
  saleInfo: saleInfo;
  accessInfo: accessInfo;
}

export interface SearchResult {
  items: Array<Book>;
  kind: string;
  totalItems: number;
}

@Injectable({
  providedIn: 'root',
  deps: [HttpClient],
})
export class GoogleBookApiService {
  constructor(private http: HttpClient) {}

  public search(query: string): Observable<SearchResult> {
    return this.http.get<SearchResult>(
      `https://www.googleapis.com/books/v1/volumes?q=${query}&maxResults=40`,
    );
  }
}
