import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { IonModal } from '@ionic/angular';
import { Book } from '../../api/google-book-api.service';
import { InAppBrowser } from '@awesome-cordova-plugins/in-app-browser';
import {
  bookIssued,
  readerUser,
  StorageService,
} from '../../services/storage/storage.service';

interface usersIssued {
  user: readerUser;
  issue: bookIssued;
}

@Component({
  selector: 'app-book-page',
  templateUrl: './book-page.component.html',
  styleUrls: ['./book-page.component.scss'],
})
export class BookPageComponent implements OnInit {
  @ViewChild(IonModal) modal: IonModal | undefined;
  @Input() item!: Book;
  @Input() trigger!: string;
  public openBrowser(url: string) {
    InAppBrowser.create(url, '_blank', {
      allowInlineMediaPlayback: 'yes',
      toolbar: 'yes',
      useWideViewPort: 'yes',
      hidden: 'no',
    });
  }
  constructor(private storage: StorageService) {}

  public httpToHttps(url: string) {
    if (url.startsWith('http://')) {
      return url.replace('http://', 'https://');
    }
    return url;
  }

  ngOnInit() {
    this.getIssuers();
  }

  public issued: Array<usersIssued> = [];
  cancel() {
    this.modal?.dismiss(null, 'cancel');
  }

  getIssuers() {
    this.storage.init().then(() => {
      this.storage.getBookIssuedByBookId(this.item.id).then((issuers) => {
        this.storage.getReaderUsers().then((users) => {
          issuers.forEach((issuer) => {
            users.forEach((user) => {
              if (user.id === issuer.userId) {
                this.issued.push({ user: user, issue: issuer });
              }
            });
          });
        });
      });
    });
  }

  parseDate(date: string) {
    return new Date(date).toLocaleDateString();
  }

  protected readonly open = open;
}
