import { Component, OnInit } from '@angular/core';
import {
  bookIssued,
  readerUser,
  StorageService,
} from '../../services/storage/storage.service';
import { Book, GoogleBookApiService } from '../../api/google-book-api.service';

interface issue {
  issue: bookIssued;
  book: Book;
  user: readerUser | null;
}

@Component({
  selector: 'app-rent-page',
  templateUrl: 'rent.page.html',
  styleUrls: ['rent.page.scss'],
})
export class RentPage implements OnInit {
  constructor(
    private storage: StorageService,
    private ApiService: GoogleBookApiService,
  ) {}

  protected booksName: Array<issue> = [];
  protected refresh() {
    this.ngOnInit();
  }
  ngOnInit() {
    this.booksName = [];
    this.storage.init().then(() => {
      this.storage.getBooksIssued().then((issues) => {
        this.storage.getReaderUsers().then((users) => {
          issues.forEach((issue) => {
            return this.ApiService.search(issue.bookId).subscribe({
              next: (book) => {
                this.booksName.push({
                  issue: issue,
                  book: book.items[0],
                  user:
                    users.find((user) => {
                      return user.id === issue.userId;
                    }) || null,
                });
              },
            });
          });
        });
      });
    });
  }

  public removeIssue(issue: bookIssued) {
    this.storage.init().then(() => {
      this.storage.removeBookIssued(issue).then(() => {
        this.refresh();
      });
    });
  }

  public dateParse(date: string) {
    let _date = new Date(date);
    return _date.toLocaleDateString();
  }

  public httpToHttps(url: string) {
    if (url.startsWith('http://')) {
      return url.replace('http://', 'https://');
    }
    return url;
  }
}
